#include <stdint.h>

#include <zephyr/kernel.h>
#include <zephyr/logging/log.h>

#include <zephyr/net/wifi.h>
#include <zephyr/net/wifi_mgmt.h>
#include <zephyr/net/net_mgmt.h>

LOG_MODULE_REGISTER(wifi_driver, LOG_LEVEL_INF);

static struct net_mgmt_event_callback wifi_mgmt_cb;

static void wifi_event_handler(struct net_mgmt_event_callback *cb,
                               uint32_t event, struct net_if *iface)
{
    if (event == NET_EVENT_WIFI_CONNECT_RESULT) {
        struct wifi_status *status = (struct wifi_status *)cb->info;
        if (status->status) {
            LOG_ERR("Wi-Fi Connection Failed (Status: %d)", status->status);
        } else {
            LOG_INF("Wi-Fi Connected Successfully!");
        }
    } else if (event == NET_EVENT_WIFI_DISCONNECT_RESULT) {
        LOG_WRN("Wi-Fi Disconnected!");
    }
}

uint8_t wifi_init(void)
{
    struct net_if *iface = net_if_get_default();
    if (!iface) {
        LOG_ERR("No default network interface found!");
        return -ENODEV;
    }

    net_mgmt_init_event_callback(&wifi_mgmt_cb, wifi_event_handler,
                                 NET_EVENT_WIFI_CONNECT_RESULT | NET_EVENT_WIFI_DISCONNECT_RESULT);
    net_mgmt_add_event_callback(&wifi_mgmt_cb);

    LOG_INF("Wi-Fi Initialized");
    return 0;
}

int8_t wifi_connect(char* ssid, char* passkey)
{
    struct net_if *iface = net_if_get_first_wifi();
    if (iface == NULL) {
        LOG_ERR("No Wi-Fi network interface found!");
        return -ENODEV;
    }

    struct wifi_connect_req_params params = {
        .ssid = ssid,
        .ssid_length = strlen(ssid),
        .psk = passkey,
        .psk_length = strlen(passkey),
        .channel = WIFI_CHANNEL_ANY,
        .security = WIFI_SECURITY_TYPE_PSK,
        .mfp = WIFI_MFP_OPTIONAL,
        .timeout = SYS_FOREVER_MS,
        .band = WIFI_FREQ_BAND_UNKNOWN,
    };

    int err = net_mgmt(NET_REQUEST_WIFI_CONNECT, iface, &params, sizeof(params));
    if (err) {
        LOG_ERR("Wi-Fi Connection Failed (Error: %d)", err);
        return err;
    }

    LOG_INF("Connecting to Wi-Fi: %s", ssid);
    return 0;
}
